
/*
 * $Id: ConnMgr.cc,v 1.14.4.3 1999/10/18 23:19:19 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "Rnd.h"
#include "ConnIdx.h"
#include "ConnMgr.h"
#include "gadgets.h"


ObjFarm<Connection> ConnMgr::TheConnFarm;
ConnIdx ConnMgr::TheConnIdx;


ConnMgr::ConnMgr(): thePconnUseLmt(0), theConnLvl(0) {
	// init static arrays (could init more than once, but that's ok)
	TheConnFarm.limit(Socket::TheMaxLevel+1);
	TheConnIdx.fdLimit(Socket::TheMaxLevel+1);
}

void ConnMgr::configure(const SockOpt &anOpt, RndDistr *aPconnUseLmt) {
	Assert(!thePconnUseLmt);
	thePconnUseLmt = aPconnUseLmt;
	theSockOpt = anOpt;
}

void ConnMgr::put(Connection *conn) {
	TheConnIdx.check(conn);

	// see if we want to keep this connection open
	if (conn->reusable())
		putIdle(conn); // may close due to external limits
	else
		close(conn);
}

// adds freshly opened connection
void ConnMgr::opened(Connection *conn) {
	conn->mgr(this);
	TheConnIdx.add(conn);
	setUseLimit(conn);
	theConnLvl++;
}

// internal method to close a connection, no checks made here
void ConnMgr::close(Connection *conn) {
	//cerr << here << "closing fd: " << conn->sock().fd() << " conn: " << (void*)conn << endl;
	//aboutToClose(conn);
	TheConnIdx.del(conn);
	conn->close();
	TheConnFarm.put(conn);
	theConnLvl--;
}

void ConnMgr::setUseLimit(Connection *conn) {
	int limit = 1; // no pconns by default
	if (thePconnUseLmt)
		limit = Max(1, (int)thePconnUseLmt->trial());
	conn->useLimit(limit);
}
