
/*
 * $Id: ContBuildIter.cc,v 1.3.2.1 1999/10/25 19:17:34 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include <iostream.h>
#include <strstream.h>

#include "IOBuf.h"
#include "EmbedContMdl.h"
#include "ContBuildIter.h"
#include "gadgets.h"

ContBuildIter::ContBuildIter() {
	reset();
}

void ContBuildIter::configure(EmbedContMdl *aModel, WrBuf *aBuf, Size aContSize) {
	theModel = aModel;
	theBuf = aBuf;
	theContSize = aContSize;
}

void ContBuildIter::reset() {
	theModel = 0;
	theBuf = 0;
	theOid = -1;
	theContSize = -1;
	theBuiltSize = 0;
	theEmbedGoal = -1;
	theEmbedCount = 0;
	theNewEmbedCount = 0;
	theEmbedDist = -1;
}

void ContBuildIter::oid(int anOid, int aSeed) {
	Assert(theModel);
	Assert(theEmbedCount == 0);
	theOid = anOid;
	theRng.seed(aSeed);
	theEmbedGoal = theModel->embedGoal(theRng);
	Assert(theEmbedGoal >= 0);
	theEmbedDist = theContSize / (theEmbedGoal+1);
}

void ContBuildIter::slice() {
	while (theContSize > theBuiltSize && !theBuf->full()) {
		// fill up to next embed tag
		const Size nextSizeGoal = theEmbedCount < theEmbedGoal ?
			Max(theBuiltSize, theEmbedDist*theEmbedCount + theEmbedDist) :
			theContSize;
		if (const Size left = nextSizeGoal - theBuiltSize)
			theBuiltSize += theBuf->appendRndUpTo(left);

		// put the embed tag
		if (theEmbedCount < theEmbedGoal) {
			if (const int sz = embed())
				theBuiltSize += sz;
			else
				break; // not enough buf space to put the tag!
		}
	}
}

// writes embedded oid tag into the buffer if there is enough space
// and we are not exceeding content length; return #bytes appended
Size ContBuildIter::embed() {
	int etype = 0;
	const int eoid = theModel->embedOid(theRng, theOid, etype, theNewEmbedCount);
	Assert(etype > 0);
	
	// XXX: rewrite this using actual IO buffer space and back-off
	// in emergency
	static char buf[32];
	ostrstream os(buf, sizeof(buf));
	os << '<' << etype << '/' << hex << eoid << '>';
	const Size sz = os.tellp();

	// check that we are not exceeding content length or buffer size
	const Size left = theContSize - theBuiltSize;
	if (sz > left)
		return theBuf->appendRndUpTo(left);

	// postpone if more space is needed
	if (theBuf->spaceSize() < sz)
		return 0;

	theBuf->append(buf, sz);

	theEmbedCount++;
	return sz;
}
