
/*
 * $Id: IntIntHash.cc,v 1.2 1999/09/17 23:54:12 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include <string.h>

#include "IntIntHash.h"
#include "gadgets.h"


IntIntHash::IntIntHash(int aCapacity): theHashCap(0) {
	theHashCap = (aCapacity + aCapacity/3 + 7) | 1;
	theIndex = new (IntIntHashItem*[theHashCap]);
	memset(theIndex, 0, sizeof(IntIntHashItem*)*theHashCap);
}

IntIntHash::~IntIntHash() {
	delete[] theIndex;
}

double IntIntHash::utilp() const {
	return Percent(theHashCnt, theHashCap);
}

bool IntIntHash::find(int key, Loc &loc) const {
	Assert(key);

	loc = theIndex + hashIdx(key);

	// search the chain
	while (*loc && (*loc)->key < key) loc = &(*loc)->next;

	return *loc && (*loc)->key == key;
}

void IntIntHash::addAt(Loc loc, int key, int val) {
	Assert(key);

	IntIntHashItem *i = getNewItem();
	i->next = *loc;
	i->key = key;
	i->val = val;
	*loc = i;

	if (!i->next && theIndex <= loc && loc < theIndex+theHashCap) {
		theHashCnt++;
		Assert(theHashCnt <= theHashCap);
	}
}

void IntIntHash::delAt(Loc loc) {
	Assert(*loc && (*loc)->key);

	IntIntHashItem *i = *loc;
	*loc = i->next;
	putOldItem(i);

	if (!*loc && theIndex <= loc && loc < theIndex+theHashCap) {
		theHashCnt--;
		Assert(theHashCnt >= 0);
	}
}
