
/*
 * $Id: RndPermut.cc,v 1.3.6.1 1999/10/14 20:57:44 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include <limits.h>

#include "Map.h"
#include "Rnd.h"
#include "String.h"
#include "RndPermut.h"

static int *ThePermTable = 0;
static int ThePermTableCap = 0;


void RndPermutConfig(int setSize, int seed) {
	delete ThePermTable;

	if (setSize > 0) {
		ThePermTable = new int[setSize];
		ThePermTableCap = setSize;
		RndPermutSeed(seed);
	} else {
		ThePermTable = 0;
		ThePermTableCap = 0;
	}
}

// feel set with "good" values
void RndPermutSeed(int seed) {
	RndGen rng(seed);
	for (int i = 0; i < ThePermTableCap; ++i)
		ThePermTable[i] = rng.ltrial();
}

inline
int PermItem(int i) {
	return i >= 0 ?
		ThePermTable[i % ThePermTableCap] :
		ThePermTable[(i + INT_MAX) % ThePermTableCap];
}

int RndPermut(int n, int m) {
	const int a = n / ThePermTableCap;
	const int b = n % ThePermTableCap;
	const int offset = PermItem(a) - m;
	return PermItem(offset + b);
}


RndGen *GenRndGen(const String &group) {
	static Map<RndGen*> gens;

	RndGen *gen = 0;
	if (!gens.find(group, gen)) {
		gen = new RndGen(RndPermut(gens.count(), group.len()));
		gens.add(group, gen);
	}
	return gen;
}

/* initialization */

int RndPermutfInit::TheUseCount = 0;

void RndPermutfInit::init() {
	RndPermutConfig(1048573); // 82,025-th prime number! (1M=1048576)
}

void RndPermutfInit::clean() {
	RndPermutConfig(0);
}
