
/*
 * $Id: ServerRep.cc,v 1.2.4.2 1999/10/22 21:41:48 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "ServerRep.h"
#include "ErrorMgr.h"
#include "OLog.h"

#include "gadgets.h"
#include "polyErrors.h"


int ServerRep::TheNewOidsPrefetchDepth = -1;
int ServerRep::TheNewOidsPerReqMax = -1;

void ServerRep::ConfigureNids(int prefetchLmt, int perReqMax) {
	TheNewOidsPrefetchDepth = prefetchLmt;
	TheNewOidsPerReqMax = perReqMax;
}

ServerRep::ServerRep(const NetAddr &anAddr): theAddr(anAddr),
	theSecOid(0), theWorkSetCap(-1), thePendCnt(0) {
	if (TheNewOidsPrefetchDepth > -1)
		theNewOids.resize(TheNewOidsPrefetchDepth);
	
}

int ServerRep::oidShortage() {
	//cerr << here << thePendCnt << " + " << theNewOids.count() << " = " << shortage << endl;
	const int shortage = theNewOids.capacity() - (thePendCnt + theNewOids.count());
	const int req_size = Min(TheNewOidsPerReqMax, shortage);
	thePendCnt += req_size;
	return req_size;
}

int ServerRep::newOid() {
	Assert(haveNewOids());
	return theNewOids.dequeue();
}

void ServerRep::noteNewOids(int oid, int askCnt, int gotCnt) {
	Assert(!gotCnt || oid > 0);

	thePendCnt -= askCnt;
	if (thePendCnt < 0)
		thePendCnt = 0;

	while (gotCnt-- > 0) {
		if (theNewOids.full()) {
			ReportError(errTooManyNewOids);
			break;
		}
		theNewOids.enqueue(oid++);
	}
}

void ServerRep::syncSrvInfo(const UniqId &aSrvId, int aSecOid) {
	if (!theId)
		theId = aSrvId;
	else
	if (theId != aSrvId) {
		ReportError(errSrvChangedWid);
		theNewOids.reset();
		theId = aSrvId;
		theSecOid = 0;
		thePendCnt = 0;
	}

	if (aSecOid > theSecOid)
		theSecOid = aSecOid;
}

void ServerRep::freezeWorkSetCap() {
	theWorkSetCap = theSecOid;
}

void ServerRep::store(OLog &log) const {
	log
		<< theAddr
		<< theId
		// << theNewOids  // not logging new oids for now
		<< theSecOid
		<< theWorkSetCap
		;
}
