
/*
 * $Id: StatCycle.cc,v 1.6.6.5 1999/11/07 05:00:33 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include <iomanip.h>

#include "StatPhase.h"
#include "StatPhaseMgr.h"
#include "Socket.h"
#include "LogComment.h"
#include "Notifier.h"
#include "PolyOLog.h"
#include "StatCycle.h"

#include "gadgets.h"
#include "globals.h"
#include "polyLogTags.h"


StatCycle::StatCycle(): StatIntvl(theRec), theNotifier(0) {
}

void StatCycle::period(Time aPeriod) {
	thePeriod = aPeriod;
}

void StatCycle::start() {
	Assert(thePeriod > 0);
	startListen();
	nextIntvl(TheClock + thePeriod);
}

void StatCycle::notifier(Notifier *aNotifier) {
	theNotifier = aNotifier;
}

void StatCycle::wakeUp(const Alarm &alarm) {
	StatIntvl::wakeUp(alarm);
	restart();
	nextIntvl(TheClock + thePeriod); // clock skew is possible
}

void StatCycle::nextIntvl(Time wakeUpTime) {
	theIntvlStart = TheClock;
	sleepTill(wakeUpTime);
}

void StatCycle::restart() {
	Assert(theIntvlStart > 0);
	theRec.theDuration = TheClock - theIntvlStart; // real duration
	if (theRec.theDuration < 0) // sometimes clock goes backwards!
		theRec.theDuration = Time(0, 0);
	TheOLog << bege(lgStatCycleRec) << theRec << ende;
	report();
	if (theNotifier)
		theNotifier->noteStats(theRec);
	theRec.restart();
}

void StatCycle::report() const {
	// note: using Comment leads to part of this info being logged twice
	// once as ascii and once as binary
	Comment(2) << "i-" << TheStatPhaseMgr->name();
	StatIntvl::report(Comment);
	Comment
		<< ' ' << setw(3) << Socket::Level()
		<< endc;
}
