
/*
 * $Id: AggrStat.h,v 1.2.6.1 1999/10/28 21:12:31 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef AGGR_STAT_H
#define AGGR_STAT_H

#include "OLog.h"
#include "ILog.h"

class ostream;

// simple aggregate statisitcs

class AggrStat {
	public:
		typedef int Val;

	public:
		AggrStat();

		void reset();
		void record(Val val);
		void store(OLog &log) const;
		void load(ILog &log);

		AggrStat &operator +=(const AggrStat &s);

		int count() const { return theCount; }
		double mean() const { return theCount ? theSum/theCount : -1; }
		double stdDev() const;
		double relDevp() const;

		ostream &print(ostream &os, const String &pfx = "") const;

	protected:
		int theCount;
		Val theMax;
		Val theMin;
		double theSum;
		double theSqSum;
};

inline
ostream &operator <<(ostream &os, const AggrStat &s) { return s.print(os); }

inline
OLog &operator <<(OLog &log, const AggrStat &s) { s.store(log); return log; }

inline
ILog &operator >>(ILog &log, AggrStat &s) { s.load(log); return log; }

#endif
