
/*
 * $Id: Client.h,v 1.23.2.10 1999/11/06 00:35:11 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_CLIENT_CLASS_H
#define PG_CLIENT_CLASS_H

#include "AlarmClock.h"
#include "Farm.h"
#include "Agent.h"
#include "CltXact.h"
#include "BcastRcver.h"

class Connection;
class CltConnMgr;
class CltCfg;
class PortMgr;
class ServerRep;
class OidRec;
class OidCache;
class RobotSym;

class Client: public Agent, public AlarmUser, public BcastRcver {
	public:
		static void LogState(OLog &log);
		static ServerRep *GetServerRep(int idx);

	public:
		Client();
		virtual ~Client();

		virtual void configure(const RobotSym *cfg, const NetAddr &aHost, bool bind2iface);
		virtual void start();

		virtual void noteXactDone(CltXact *x);
		virtual void noteEmbedded(CltXact *x, const OidRec &embedOidRec);

	protected:
		static bool cAddServer(const NetAddr &addr, bool forceAdd);
		static void FreezeSrvWorkSetCap();

		PortMgr *getPortMgr(bool bind2iface);
		void scanServers(const RobotSym *rs, CltCfg *cfg);
		CltCfg *sharedConfig(const RobotSym *rs);

		void scheduleLaunch(Time lastLaunch);
		bool launch(CltXact *x);
		bool tryLaunch(CltXact *x = 0);
		CltXact *genXact(const OidRec &oidRec);
		CltXact *genXact();

		void genOid(OidRec &oidRec);
		void genOid2(OidRec &oidRec);

		virtual void noteInfoEvent(BcastChannel *ch, InfoEvent ev);
		virtual void noteLogEvent(BcastChannel *ch, OLog &log);

	protected:
		static ObjFarm<CltXact> TheXacts;
		static Array<PortMgr*> ThePortMgrs;  // port managers for all aliases
		static Array<ServerRep*> TheServers; // all servers known to clients
		static int TheClientCnt;             // global client counter
		static int TheTouchAllSrvCnt;        // clients that touched all srvrs
		static int TheShrdWorkSetCap;        // cap for all servers together

		Array<int> thePrivOids;       // per-origin private oid levels
		Array<int> thePrivWorkSetCap; // per-origin private work set limits
		Array<CltXact*> theLaunchDebts; // postponed requests

		CltCfg *theCfg;               // client configuration (maybe shared)
		CltConnMgr *theConnMgr;       // one for all servers
		OidCache *theCache;           // robot cache simulation
		int theTouchedSrvCnt;         // servers with private or secured oids

		int theCcXactLvl;      // outstanding concurrent xactions level
		int theLaunchCount;    // total number of xaction launches
		int theExtraLaunchLvl; // uncompensated out-of-order launches
};

#endif
