
/*
 * $Id: Oid.h,v 1.9.2.2 1999/10/23 22:54:46 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_OID_RECORD_CLASS_H
#define PG_OID_RECORD_CLASS_H

#include "UniqId.h"

// storage for oid properties
class OidRec {
	public:
		OidRec() { reset(); }

		void reset();

		bool direct() const { return tid == -1; }
		bool embedded() const { return tid > 0; }

		ostream &print(ostream &os) const;
		ostream &printOid(ostream &os) const;
		ostream &printUriPath(ostream &os) const;
		ostream &printFlags(ostream &os) const;

		void store(OLog &ol) const;
		void load(ILog &il);

	public:
		UniqId wid;     // server world id (cached)

		int sidx;       // server index
		int tid;        // object content type id
		int oid;        // object id

		bool repeat;    // was requested before
		bool cachable;  // reply is cachable
		bool hit;       // second-hand reply
		bool foreign;   // foreign request or reply
};


inline
ostream &operator <<(ostream &os, const OidRec &or) {
	return or.printOid(os);
}

inline
OLog &operator <<(OLog &ol, const OidRec &or) {
	or.store(ol);
	return ol;
}

inline
ILog &operator >>(ILog &il, OidRec &or) {
	or.load(il);
	return il;
}

#endif
