
/*
 * $Id: PolyApp.h,v 1.8.4.5 1999/11/08 04:29:28 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_APPLICATION_H
#define PG_APPLICATION_H

#include "NetAddr.h"
#include "String.h"
#include "Array.h"
#include "CmdLine.h"
#include "FileScanner.h"
#include "BcastRcver.h"

class OLog;
class Agent;
class AgentSym;
class AgentCfg;
class SizeOpt;

// holds information required during agent configuration process
struct AgentCfg {
    AgentSym *sym;
    NetAddr *host;

    AgentCfg();
    AgentCfg(AgentSym *aSym, const NetAddr &aHost);
};


// common interface for polyclt and polysrv applications
class PolyApp: public FileScanTicker, public BcastRcver {
	public:
		PolyApp();
		virtual ~PolyApp();

		virtual void noteMsgStrEvent(BcastChannel *ch, const char *msg);
		virtual void begCycle(int readyCount);
		virtual void endCycle();
		virtual bool tick();

		int run(int argc, char *argv[]);
		void flushState();

	protected:
		bool handleCmdLine(int argc, char *argv[]);
		virtual void configure();
		virtual void makeAgents();
		virtual void reportCfg();
		void reportRUsage() const;
		
		void configureBinLog(OLog &log, const String &fname, SizeOpt &bufSz);
		void configureLogs(int prec);
		void buildSchedule();
		void parseConfigFile(const String &fname);
		void step();

		void getIfaces();
		void getFakeIfaces();
		int countAddress(const NetAddr &addr) const;
		bool moreThanOneAddr() const;
		
		virtual void getOpts(Array<OptGrp*> &opts);
		virtual void wasIdle(Time tout);
		virtual Agent *makeAgent(const AgentCfg &cfg) = 0;
		virtual void logState(OLog &log);

	protected:
		CmdLine theCmdLine;

		SchArray<in_addr> theIfaces; // list of interface or fake addresses
		Array<AgentCfg> theLocalCfgs;
		Array<Agent*> theLocals;
		String theAgentType;
		bool agentMustBind;

		Time theIdleBeg;  // the start of an idle period
		Time theIdleEnd;  // beg + idle_tout
		bool isIdle;

		int theTickCount;  // see PolyApp::tick()
		int theStateCount; // number of states [flushed]
};

#endif
