
/*
 * $Id: PortMgr.h,v 1.6 1999/09/26 23:36:22 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_PORT_MANAGER_H
#define PG_PORT_MANAGER_H

#include "LevelStat.h"
#include "BitMask.h"
#include "NetAddr.h"

class Socket;

// manages allocation of network ports
// see EphPortMgr and ExpPortMgr for concrete classes
class PortMgr {
	public:
		static const LevelStat &BoundLvl() { return TheBoundLvl; }

	public:
		PortMgr(const NetAddr &anAddr);
		virtual ~PortMgr() {}

		const NetAddr &addr() const { return theAddr; }

		int bind(Socket &s); // returns port number
		void release(int port, bool good);

	protected:
		virtual int allocPort(Socket &s) = 0;
		virtual void freePort(int port, bool good) = 0;

	protected:
		static LevelStat TheBoundLvl; // bound ports level

	protected:
		NetAddr theAddr;              // local address to bind to
};

#endif
