
/*
 * $Id: PglCtx.cc,v 1.4 1999/09/07 17:47:41 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"
#include "xparser.h"

#include "PglCtx.h"
#include "PglRec.h"
#include "PglRecSym.h"
//#include "pglSyms.h"

PglCtx *PglCtx::TheRootCtx = new PglCtx(0);


PglCtx::PglCtx(const String &aName, PglCtx *aParent): 
	theName(aName), theParent(aParent) {
}

PglCtx::~PglCtx() {
}

SynSymTblItem **PglCtx::valp(const String &key) {
	// if named, try searching with our name as prefix
	if (theName) { 
		String fullName = theName;
		fullName += ".";
		fullName += key;
		if (SynSymTblItem **i = memberp(fullName))
			return i;
	}

	// now search without a prefix if prefix does not help

	// if it is a complex name, lookup rules change
	if (key.chr('.'))
		return memberp(key);

	// standard recursive lookup
	return valpR(key);
}

// recursive valp
SynSymTblItem **PglCtx::valpR(const String &key) {
	Assert(!key.chr('.'));

	// check local namespace
	if (SynSymTblItem **i = SynSymTbl::valp(key))
		return i;

	// ask parent
	return theParent ? theParent->valp(key) : 0;
}

// searches for a symbol that corresponds to a composite name
SynSymTblItem **PglCtx::memberp(const String &name) {
	const char *tail = name.chr('.');
	Assert(tail);
	const String key = name(0, tail - name); // first component

	// find ancor
	SynSymTblItem **ip = valpR(key);
	if (!ip)
		return 0;

	SynSymTblItem *i = *ip;
	// ancor must point to a RecSym
	if (!(i && i->sym() && i->sym()->isA("Rec")))
		return 0;

	RecSym &s = (RecSym &)i->sym()->cast("Rec");
	Assert(s.rec());

	return s.rec()->valp(tail + 1);
}

int PglCtx::nestDepth() const {
	return 1 + (theParent ? theParent->nestDepth() : 0);
}

void PglCtx::noteAdd(int idx) {
	theItems[idx].theVal->ctx(this);
}

void PglCtx::noteDel(int idx) {
	theItems[idx].theVal->ctx(0);
}

void PglCtx::report(ostream &os, const String &pfx) const {
	if (theName)
		os << pfx << theName << ':' << endl;

	for (int i = 0; i < count(); ++i) {
		if (valAt(i)) {

			os << pfx << valAt(i)->type() << ' ' << valAt(i)->name() << " = ";
			if (valAt(i)->sym())
				valAt(i)->sym()->print(os, pfx);
			else
				os << "undef()";
			os << ';' << endl;
		}
	}
}
