
/*
 * $Id: PhaseSym.cc,v 1.2.6.1 1999/10/22 23:28:01 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include "PglRec.h"
#include "PglStringSym.h"
#include "GoalSym.h"
#include "PhaseSym.h"


PhaseSym::PhaseSym(): RecSym("Phase", new PglRec) {
	theRec->bAdd("string", "name", 0);
	theRec->bAdd("Goal", "goal", new GoalSym);

	theRec->bAdd("float", "load_factor_beg", 0);
	theRec->bAdd("float", "load_factor_end", 0);

	theRec->bAdd("bool", "log_stats", 0);
}

PhaseSym::PhaseSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
}

bool PhaseSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "Phase";
}

SynSym *PhaseSym::clone(const String &type) const {
	if (isA(type))
		return new PhaseSym(this->type(), theRec->clone());
	return RecSym::clone(type);
}

void PhaseSym::name(const String &aName) {
	SynSymTblItem *ni = 0;
	Assert(theRec->find("name", ni));
	delete ni->sym();
	ni->sym(new StringSym(aName));
}

String PhaseSym::name() const {
	return getString("name");
}

GoalSym *PhaseSym::goal() const {
	SynSymTblItem *gi = 0;
	Assert(theRec->find("goal", gi));
	Assert(gi->sym());
	return &(GoalSym&)gi->sym()->cast("Goal");
}

bool PhaseSym::loadFactorBeg(double &lf) const {
	return getDouble("load_factor_beg", lf);
}

bool PhaseSym::loadFactorEnd(double &lf) const {
	return getDouble("load_factor_end", lf);
}

bool PhaseSym::logStats(bool &ls) const {
	return getBool("log_stats", ls);
}
