
/*
 * $Id: ServerSym.cc,v 1.8.2.2 1999/10/25 19:17:40 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include "String.h"
#include "PglIntSym.h"
#include "PglRec.h"
#include "PglArraySym.h"
#include "PglStringSym.h"
#include "ServerSym.h"


ServerSym::ServerSym(): AgentSym("Server") {
	theRec->bAdd("int", "accept_lmt", 0);
	theRec->bAdd("Content[]", "contents", 0);
	theRec->bAdd("Content[]", "direct_access", 0);
}

ServerSym::ServerSym(const String aType, PglRec *aRec):	AgentSym(aType, aRec) {
}

bool ServerSym::isA(const String &type) const {
	return AgentSym::isA(type) || type == "Server";
}

SynSym *ServerSym::clone(const String &type) const {
	if (isA(type))
		return new ServerSym(this->type(), theRec->clone());
	return AgentSym::clone(type);
}

bool ServerSym::acceptLmt(int &lmt) const {
	SynSymTblItem *lmti = 0;
	Assert(theRec->find("accept_lmt", lmti));
	if (lmti->sym())
		lmt = ((const IntSym&)lmti->sym()->cast("int")).val();
	return lmti->sym();
}

bool ServerSym::contents(Array<ContentSym*> &ccfgs, Array<double> &cprobs) const {
	SynSymTblItem *conti = 0;
	Assert(theRec->find("contents", conti));
	if (!conti->sym())
		return false; // undefined

	ArraySym &a = (ArraySym&)conti->sym()->cast("Array");
	a.copyProbs(cprobs);
	ArraySymExport(a, "Content", ccfgs);
	return true;
}

bool ServerSym::directAccess(Array<ContentSym*> &dacfgs) const {
	SynSymTblItem *dai = 0;
	Assert(theRec->find("direct_access", dai));

	// use "contents" array if "direct_access" is not defined
	if (!dai->sym())
		Assert(theRec->find("contents", dai));
	if (!dai->sym())
		return false; // undefined

	ArraySym &a = (ArraySym&)dai->sym()->cast("Array");
	ArraySymExport(a, "Content", dacfgs);
	return true;
}
