
/*
 * $Id: PglArraySym.h,v 1.5.6.3 1999/11/07 02:48:25 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PGL_ARRAY_SYMBOL_H
#define PGL_ARRAY_SYMBOL_H

#include "PglListSym.h"

class RndDistr;
class String;

// list of symbols of a specified type
class ArraySym: public ListSym {
	public:
		ArraySym(const String &anItemType);

		virtual bool isA(const String &type) const;
		virtual SynSym *clone(const String &type) const;

		const String &itemType() const { return theItemType; }

		double prob(int idx) const { return theProbs[idx]; }
		void addProb(const SynSym &s, double prob);

		// rnd distr of array idxes that matches item probabilities
		RndDistr *makeSelector(const String &name);
		void copyProbs(Array<double> &res) const;

		virtual ostream &print(ostream &os, const String &pfx = 0) const;

	protected:
		virtual void addFailure(const SynSym &s);
		virtual bool cadd(const SynSym &s);

		void prob(int idx, double p);
		void syncProbs();
		void makeProbs();
		void makeProbs(Array<double> &res) const;
		void checkProbs() const;
		double statProbs() const;

	protected:
		String theItemType;
		Array<double> theProbs; // member selection probabilities

		mutable int theUndefProbCnt; // items with no probs (cached)
		mutable bool warnedBadProbs;
};


/* template to transform array symbol into arrays of symbols/items
 * the only reason to use a template is to avoid a[i] casting
 * think: maybe the array symbol should be a template instead??
 */

template <class Item>
int ArraySymExport(const ArraySym &as, const char *typeName, 
	Array<Item*> &arr, Array<double> *probs = 0) {

	arr.stretch(as.count());
	if (probs)
		probs->stretch(as.count());
	for (int i = 0; i < as.count(); ++i) {
		Item *item = &((Item&)as[i]->cast(typeName));
		arr.append(item);
		if (probs)
			probs->append(as.prob(i));
	}
	return arr.count();
}

#endif
