
/*
 * $Id: gadgets.cc,v 1.8.2.2 1999/10/26 21:09:30 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "xstd.h"

#include <limits.h>
#include <string.h>
#include <sys/time.h>
#include <sys/resource.h>

#include <iostream.h>

#include "Assert.h"
#include "gadgets.h"



char *StrDup(const char *str, int len) {

	if (!str) return 0;

	char *copy = new char [len+1];
	memcpy(copy, str, len);
	copy[len] = '\0';

	return copy;
}

int XGCD(int a, int b) {
	return b ? XGCD(b, a % b) : a;
}

// strNchr(), sort of
const char *StrBoundChr(const char *s, char c, const char *eos) {
	while (s < eos) {
		if (*s == c)
			return s;
		else
			++s;
	}
	return 0;
}

bool isInt(const char *s, int &i, const char **p, int base) {
	if (s) {
		char *ptr = 0;
		const int h = (int) strtol(s, &ptr, base);
		if (ptr != s && ptr) {
			i = h;
			if (p) *p = ptr;
			return true;
		}
	}
	return false;
}

bool isNum(const char *s, double &d, const char **p) {
	if (s) {
		char *ptr = 0;
		const double h = strtod(s, &ptr);
		if (ptr != s) {
			d = h;
			if (p) *p = ptr;
			return true;
		}
	}
	return false;
}

bool isDbl(const char *s, double &d, const char **p) {
	double dd;
	const char *pp;
	if (!isNum(s, dd, &pp))
		return false; 

	Assert(pp);

	// check that it is not an integer (has '.' or 'e')
	const char *h = strchr(s, '.');
	bool dbl = h && h < pp;
	if (!dbl) {
		h = strchr(s, 'e');
		dbl = h && h < pp;
	}

	if (dbl) {
		d = dd;
		if (p) *p = pp;
	}
	return dbl;
}

int xatoi(const char *s, int def) { 
	isInt(s, def);
	return def;
}

void configureStream(ostream &os, int prec) {
	os.precision(prec);
	os.setf(ios::fixed, ios::floatfield);
	os.setf(ios::showpoint, ios::basefield);
}

static
int rlimit2int(rlim_t rl) {
#ifdef RLIM_INFINITY
	if (rl == RLIM_INFINITY)
		return INT_MAX;
#endif
	const int n = (int)rl;
	Should(INT_MIN <= n && n <= INT_MAX);
	return MiniMax(INT_MIN, n, INT_MAX);
}

int GetCurFD() {
	struct rlimit rl;
	if (getrlimit(RLIMIT_NOFILE, &rl) < 0)
		return -1;
	return rlimit2int(rl.rlim_cur);
}

int GetMaxFD() {
	struct rlimit rl;
	if (getrlimit(RLIMIT_NOFILE, &rl) < 0)
		return -1;
	return rlimit2int(rl.rlim_max);
}

int SetMaxFD(int maxFD) {
	struct rlimit rl;
	rl.rlim_max = Min(maxFD, GetMaxFD());
	rl.rlim_cur = rl.rlim_max;
	if (rl.rlim_cur > 0)
		Assert(setrlimit(RLIMIT_NOFILE, &rl) >= 0);
	return rlimit2int(rl.rlim_cur);
}
