
/*
 * $Id: getIfAddrs.cc,v 1.2.6.1 1999/10/01 19:06:21 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "xstd.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>
#include <arpa/inet.h>

#include <iostream.h>

#include "Assert.h"
#include "getIfAddrs.h"
#include "gadgets.h"

static int ifreqSize(const ifreq *req);

/*
 * We are trying to get a list of all interface addresses using
 * Socket::getIfConf.  There are two problems here. First, we do not know how
 * big the array buffer should be and not all (none?) OSes can tell us. Second,
 * the algorithm to calculate the ifreq size differs a lot among the OSes
 */
bool GetIfAddrs(Array<in_addr> &addrs, const char *ifname) {
	Socket s;
	if (!s.create(AF_INET, SOCK_DGRAM, 0))
		return false;

	struct ifconf cfg;
	int buf_len = 8 * 1024;
	cfg.ifc_buf = 0;

	// [re]allocate buffers until we get errors or the answer fits
	bool res = true;
	while (res) {
		delete[] cfg.ifc_buf;
		cfg.ifc_len = buf_len;
		cfg.ifc_buf = new char[cfg.ifc_len];

		res = s.getIfConf(cfg);
		if (res && cfg.ifc_len <= buf_len/2) // enough buf space
			break;
		buf_len *= 2;
	}
	
	for (int off = 0; res && off < cfg.ifc_len;) {
		const ifreq *req = (const ifreq *)(cfg.ifc_buf + off);

		// calculate the position of the next req struct
		off += ifreqSize(req);

		if (req->ifr_addr.sa_family != AF_INET)
			continue;

		if (ifname && strncmp(ifname, req->ifr_name, sizeof(req->ifr_name)))
			continue;

		addrs.append(((const sockaddr_in &)req->ifr_addr).sin_addr);
	}

	s.close();
	delete[] cfg.ifc_buf;

	return res;
}


// calculate the actual size of ifreq structure
static
int ifreqSize(const ifreq *req) {
	// note: there are many broken cases not covered here;
	// see what /usr/src/contrib/bind/bin/named/ns_main.c has to do

#if defined HAVE_SA_LEN
	return Max(sizeof(*req), req->ifr_addr.sa_len + sizeof(req->ifr_name));
#else
	return sizeof(*req);
#endif
}
